@@ -1,3 +1,5 @@
+import json
+
 import pytest
 
 from mmap_ninja import generic
@@ -62,3 +64,25 @@ def test_from_generator(tmp_path, n):
     for i in range(n):
         assert str(i) == memmap[i]
     generic.open_existing(tmp_path / "strings_memmap")
+
+
+def test_open_empty(tmp_path):
+    memmap = StringsMmap(tmp_path / "strings_memmap")
+    assert len(memmap) == 0
+    memmap.append("Something")
+    assert len(memmap) == 1
+
+
+def test_index_error(tmp_path):
+    memmap = StringsMmap(tmp_path / "strings_memmap")
+    print(memmap)
+    with pytest.raises(IndexError):
+        memmap[0]
+
+
+def test_json_wrapper(tmp_path):
+    memmap = StringsMmap(tmp_path / "strings_memmap")
+    memmap.append(json.dumps({"asdasd": "as"}))
+
+    memmap = generic.open_existing(tmp_path / "strings_memmap", wrapper_fn=json.loads)
+    print(memmap[0])
